package cz.zoubelu.service.impl;

import com.google.common.collect.Lists;
import cz.zoubelu.cache.Cache;
import cz.zoubelu.codelist.SystemApp;
import cz.zoubelu.domain.Application;
import cz.zoubelu.domain.ConsumeRelationship;
import cz.zoubelu.domain.Method;
import cz.zoubelu.model.TimeRange;
import cz.zoubelu.repository.ApplicationRepository;
import cz.zoubelu.repository.MethodRepository;
import cz.zoubelu.repository.RelationshipRepository;
import cz.zoubelu.service.DynamicEntityProvider;
import cz.zoubelu.utils.DateUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Map;

/**
 * Created by zoubas.
 */
@Service
public class DynamicEntityProviderImpl implements DynamicEntityProvider {
    private final Logger log = Logger.getLogger(getClass());

    @Autowired
    private ApplicationRepository applicationRepo;

    @Autowired
    private Cache cache;

    @Autowired
    private RelationshipRepository relationshipRepo;

    @Autowired
    private MethodRepository methodRepository;

    /**
     * DATABASE LOOKS UP FOR ENTITY
     */
    //getApplication and getMethod methods could be merged into generic or general one.
    public Application getApplication(SystemApp system) {
        if (cache.contains(system)) {
            return cache.get(system);
        } else {
            Application app = applicationRepo.findBySystemId(system.getId());
            if (app != null) {
                cache.cacheApplication(app);
                return app;
            } else {
                return createApplication(system);
            }
        }
    }

    public Method getConsumedMethod(Application providingApp, String name, Integer version) {

        if (cache.contains(providingApp, name, version)) {
            return cache.get(providingApp, name, version);
        } else {
            Method method = methodRepository.findProvidedMethod(providingApp, name, version);
            if (method != null) {
                cache.cacheMethod(providingApp, method);
                return method;
            } else {
                return createMethod(providingApp, name, version);
            }
        }
    }

    public void createConsumeRelation(Application consumer, Method method) {
        ConsumeRelationship relationship = null;

        if (cache.contains(consumer, method)) {
            relationship = cache.get(consumer, method);
        } else {
            relationship = relationshipRepo.findRelationship(consumer, method);
        }

        if (relationship != null) {
            cache.cacheRelation(relationship);
            log.debug(String.format("Relationship: %s CONSUMES -> %s, already exists, incrementing total usage by 1.", consumer.getName(), method.getName()));
            relationship.setTotalUsage(relationship.getTotalUsage() + 1);
        } else {
            createRelationship(consumer, method);
        }
    }

    private void increaseDayUsage(Map<String, Long> dailyUsage, TimeRange timeRange) {
        String actualDate = DateUtils.getDateSuffixFromTimeRange(timeRange);
        if (dailyUsage.get(actualDate) != null) {
            dailyUsage.put(actualDate, dailyUsage.get(actualDate) + 1L);
        } else {
            dailyUsage.put(actualDate, 1L);
        }
    }

    /**
     * PERSISTING CACHED RELATIONS INTO GRAPH
     */
    public void persistCachedRelations() {
        log.info("Saving cached relations into the database.");
        relationshipRepo.save(cache.getRelations());
        log.info("Saved.");
    }

    @Override
    public void clearCache() {
        cache.clearCache();
        log.info("Cache cleared.");
    }

    /**
     * DYNAMIC CREATION METHODS
     */
    private Application createApplication(SystemApp system) {
        log.info(String.format("Creating application: %s with systemID: %s", system.getName(), system.getId()));

        Application app = new Application(system.getName(), system.getId(), new ArrayList<Method>());
        cache.cacheApplication(app);

        return applicationRepo.save(app);
    }

    private Method createMethod(Application providingApp, String name, Integer version) {
        Method method = new Method(name, version);

        log.info(String.format("Creating method: %s, version: %s for app %s.", method.getName(), method.getVersion(),
                providingApp.getName()));

        if (providingApp.getProvidedMethods() != null) {
            providingApp.getProvidedMethods().add(method);
        } else {
            providingApp.setProvidedMethods(Lists.newArrayList(method));
        }
        applicationRepo.save(providingApp);
        cache.cacheMethod(providingApp, method);
        log.info(String.format("Application: %s successfully saved.", providingApp.getName()));
        return method;
    }

    private void createRelationship(Application consumer, Method method) {
        ConsumeRelationship consumeRelationship = new ConsumeRelationship(consumer, method);
        log.info(String.format("Creating new relationship: %s CONSUMES -> %s.", consumer.getName(), method.getName()));

        if (consumer.getConsumeRelationship() != null) {
            consumer.getConsumeRelationship().add(consumeRelationship);
        } else {
            consumer.setConsumeRelationship(Lists.newArrayList(consumeRelationship));
        }
        applicationRepo.save(consumer);

        cache.cacheRelation(consumeRelationship);
    }
}
